#include <QTRSensors.h>
#define NUM_SENSORS             8  // number of sensors used
#define NUM_SAMPLES_PER_SENSOR  4  // average 4 analog samples per sensor reading


#define KP 0.08
#define KD 0.3
#define KI 0.0005

#define HIST_COUNT 500
#define M1_DEFAULT_SPEED 255
#define M2_DEFAULT_SPEED 255
#define M1_MAX_SPEED 255
#define M2_MAX_SPEED 255
#define M1_MIN_SPEED -30
#define M2_MIN_SPEED -30

int hist[HIST_COUNT];
unsigned int hist_pos = 0;

struct MOTOR  
{
  int in1;      // INPUT1
  int in2;      // INPUT2
  int enable;   // ENABLE
};

MOTOR MOTOR1 = { 7, 8, 10 };
MOTOR MOTOR2 = { 3, 2, 9 };

QTRSensorsAnalog qtra((unsigned char[]) {0, 1, 2, 3, 4, 5, 6, 7}, 
  NUM_SENSORS, NUM_SAMPLES_PER_SENSOR);


/*QTRSensorsAnalog qtra((unsigned char[]) {0, 3, 7}, 
  3, NUM_SAMPLES_PER_SENSOR);*/
  
unsigned int sensorValues[NUM_SENSORS];

void setup()
{
  Serial.begin(9600);
  pinMode(MOTOR1.in1, OUTPUT);
  pinMode(MOTOR1.in2, OUTPUT);
  pinMode(MOTOR1.enable, OUTPUT);
  pinMode(MOTOR2.in1, OUTPUT);
  pinMode(MOTOR2.in2, OUTPUT);
  pinMode(MOTOR2.enable, OUTPUT);
  pinMode(12, INPUT_PULLUP);


  delay(500);
  pinMode(13, OUTPUT);
  digitalWrite(13, HIGH);
  for (int i = 0; i < HIST_COUNT; i++)
  {
    hist[i] = 0;
  }
  for (int i = 0; i < 200; i++) 
  {
    qtra.calibrate(); 
  }
  digitalWrite(13, LOW);  

  for (int i = 0; i < NUM_SENSORS; i++)
  {
    Serial.print(qtra.calibratedMinimumOn[i]);
    Serial.print(' ');
  }
  Serial.println();
  
  for (int i = 0; i < NUM_SENSORS; i++)
  {
    Serial.print(qtra.calibratedMaximumOn[i]);
    Serial.print(' ');
  }
  
  Serial.println();
  Serial.println();
  Serial.println("Wait for button...");
  
  while(digitalRead(12) != 0)
    delay(50);
}


int lastError = 0;
int  last_proportional = 0;
int integral = 0;

void loop()
{
  unsigned int sensors[NUM_SENSORS];
  int position = qtra.readLine(sensors);
  int error = position - 500 * (NUM_SENSORS-1);

  if(hist_pos == HIST_COUNT)
    hist_pos = 0;
  hist[hist_pos] = error;
  hist_pos++;

  double hist_err = 0;
  for (int i = 0; i < HIST_COUNT; i++)
  {
    hist_err += hist[i];
  }
  
  double correction = (double)KP * (double)error + (double)KD * (double)(error - lastError) + (double)KI * (double)hist_err;
  lastError = error;
  
  int leftMotorSpeed = M1_DEFAULT_SPEED + (int)correction;
  int rightMotorSpeed = M2_DEFAULT_SPEED - (int)correction;
  Serial.println(correction);
  set_motors(leftMotorSpeed, rightMotorSpeed);
}

  
void set_motors(int motor1speed, int motor2speed)
{
  if (motor1speed > M1_MAX_SPEED) motor1speed = M1_MAX_SPEED; 
  if (motor2speed > M2_MAX_SPEED) motor2speed = M2_MAX_SPEED; 
  if (motor1speed < M1_MIN_SPEED) motor1speed = M1_MIN_SPEED;
  if (motor2speed < M2_MIN_SPEED) motor2speed = M2_MIN_SPEED; 
  
 set_motor(&MOTOR1, motor1speed);
 set_motor(&MOTOR2, motor2speed);
}

void set_motor(MOTOR *m, int speed) {
  digitalWrite(m->in1, speed >= 0 ? HIGH : LOW);
  digitalWrite(m->in2, speed < 0 ?  HIGH : LOW);
  analogWrite(m->enable, speed);  
}

